
/*
* Copyright (C)  2012 - Michael Baudin
* Copyright (C)  2001 - Bruno Pincon
* Copyright (C) ENPC - jpc@cermics.enpc.fr
*
* This file must be used under the terms of the CeCILL.
* This source file is licensed as described in the file COPYING, which
* you should have received as part of this distribution.  The terms
* are also available at
* http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/

#include <string.h>
#include <math.h>

// From Distfun:
#include "gwsupport.h"
#include "cdflib.h"
#include "gw_distfunrnd.h"
#include "gw_distfunrnd_support.h" 

// From Scilab:
#include "api_scilab.h"
#include "stack-c.h"

/**************************************************
Y=distfun_rndgeom(pr,m,n) 

generates random variates from the geometric distribution with parameter pr : 
number of Bernouilli trials (with probability succes of pr) until a succes is met. 
pr must be in (0,1].

Y contains positive real numbers with integer values, with are the "number of trials to get a success".
***********************************************************************/

int sci_distfun_rndgeom(char *fname,unsigned long fname_len)
{
	int rRows,rCols;
	int readFlag;
	int ma, na;

	double * pa = NULL;

	CheckInputArgument(pvApiCtx,3,3);
	CheckOutputArgument(pvApiCtx,1,1);

    // Arg #1 : pr
	readFlag = gwsupport_GetRealMatrixOfDoubles( fname, 1, &pa, &ma, &na);
	if ( readFlag == GWSUPPORT_ERROR)
	{ 
		return 0;
	}
	readFlag = gwsupport_CheckDoubleMatrixInRange (fname, 1, pa, ma , na, 1.3e-307, 1.0);
	if ( readFlag == GWSUPPORT_ERROR)
	{ 
		return 0;
	}
	// Get Arg #2 = M, Arg #3 = N
	readFlag = distfun_GetMN(fname, 2, 3, &rRows, &rCols);
	if ( readFlag == GWSUPPORT_ERROR)
	{ 
		return 0;
	}
    // Compute the result (if necessary, expand the argument)
	readFlag = distfun_computeRandgenA (fname, ma, na, pa, rRows, rCols, cdflib_geornd);
	if ( readFlag == GWSUPPORT_ERROR)
	{ 
		return 0;
	}
	LhsVar(1) = Rhs+1;

	return 0;
}
