
/*
* Copyright (C)  2012 - Michael Baudin
* Copyright (C)  2001 - Bruno Pincon
* Copyright (C) ENPC - jpc@cermics.enpc.fr
*
* This file must be used under the terms of the CeCILL.
* This source file is licensed as described in the file COPYING, which
* you should have received as part of this distribution.  The terms
* are also available at
* http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
*
*/

#include <string.h>
#include <math.h>

// From Distfun:
#include "gwsupport.h"
#include "cdflib.h"
#include "gw_distfunrnd.h"
#include "gw_distfunrnd_support.h" 
#include "unifrng.h"

// From Scilab:
#include "api_scilab.h"
#include "Scierror.h"
#include "localization.h"
#include "stack-c.h"

/**************************************************
Y=distfun_rnduin(Low,High,m,n) 

generates random integers uniformly distributed between Low and High (included). 

High and Low must be integers such that (High-Low+1) < 2,147,483,561.
***********************************************************************/
int mygenintrange (double a, double b, double *x)
{
	int status;
	*x=unifrng_generateIntegerInRange(a, b);
	status = CDFLIB_OK;
	return status;
}

int sci_distfun_rnduin(char *fname,unsigned long fname_len)
{
	int ma, na;
	int mb, nb;
	int rRows,rCols;
	int i;
	int readFlag;
	
	double * pa = NULL;
	double * pb = NULL;

	CheckInputArgument(pvApiCtx,4,4);
	CheckOutputArgument(pvApiCtx,1,1);

    // Arg #1 : a
	readFlag = gwsupport_GetRealMatrixOfDoubles( fname, 1, &pa, &ma, &na);
	if ( readFlag == GWSUPPORT_ERROR)
	{ 
		return 0;
	}
	readFlag = gwsupport_CheckDoubleMatrixHasIntegerValue (fname, 1, pa, ma, na );
	if ( readFlag == GWSUPPORT_ERROR)
	{ 
		return 0;
	}
    // Arg #2 : b
	readFlag = gwsupport_GetRealMatrixOfDoubles( fname, 2, &pb, &mb, &nb);
	if ( readFlag == GWSUPPORT_ERROR)
	{ 
		return 0;
	}
	readFlag = gwsupport_CheckDoubleMatrixHasIntegerValue (fname, 2, pb, mb, nb );
	if ( readFlag == GWSUPPORT_ERROR)
	{ 
		return 0;
	}
	// Check that size(a)==size(b)
	readFlag = gwsupport_CheckSize ( fname, 2, ma, na, mb, nb);
	if ( readFlag == GWSUPPORT_ERROR)
	{ 
		return 0;
	}
	// Check that (b-a+1) <= 2147483561
	for ( i=0 ; i < ma*na ; i++)
	{
		if ( (*(pb+i)-*(pa+i)+1) > 2147483561 )
		{
			Scierror(999,_("%s: Wrong value for input argument #%d: ""%s"" expected .\n"),fname,1,"(b-a+1) <= 2147483561");
			return 0;
		}
	}
	// Get Arg #3 = M, Arg #4 = N
	readFlag = distfun_GetMN(fname, 3, 4, &rRows, &rCols);
	if ( readFlag == GWSUPPORT_ERROR)
	{ 
		return 0;
	}
    // Compute the result (if necessary, expand the arguments a and b)
	readFlag = distfun_computeRandgenAB (fname, ma, na, pa, mb, nb, pb, rRows, rCols, mygenintrange );
	if ( readFlag == GWSUPPORT_ERROR)
	{ 
		return 0;
	}
	LhsVar(1) = Rhs+1;
	return 0;
}
