/*
 * Copyright (C) 2012 - Michael Baudin
 *
 * This file must be used under the terms of the CeCILL.
 * This source file is licensed as described in the file COPYING, which
 * you should have received as part of this distribution.  The terms
 * are also available at
 * http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
 *
 */

#include "cdflib.h"
#include "cdflib_private.h"

int cdflib_genmn(double *parm, double *x, double *work)
{
	int i__1, i__2;

	static int i__, j, p;
	static double ae;
	static int icount;

	/* Parameter adjustments */
	--work;
	--x;
	--parm;

	/* Function Body */
	p = (int) parm[1];

	/*     Generate P independent normal deviates - WORK ~ N(0,1) */

	i__1 = p;
	for (i__ = 1; i__ <= i__1; ++i__) {
		work[i__] = cdflib_snorm();
	}
	i__1 = p;
	for (i__ = 1; i__ <= i__1; ++i__) {

		/*     PARM (P+2 : P*(P+3)/2 + 1) contains A, the Cholesky */
		/*      decomposition of the desired covariance matrix. */
		/*          trans(A)(1,1) = PARM(P+2) */
		/*          trans(A)(2,1) = PARM(P+3) */
		/*          trans(A)(2,2) = PARM(P+2+P) */
		/*          trans(A)(3,1) = PARM(P+4) */
		/*          trans(A)(3,2) = PARM(P+3+P) */
		/*          trans(A)(3,3) = PARM(P+2-1+2P)  ... */

		/*     trans(A)*WORK + MEANV ~ N(MEANV,COVM) */

		icount = 0;
		ae = 0.f;
		i__2 = i__;
		for (j = 1; j <= i__2; ++j) {
			icount = icount + j - 1;
			ae += parm[i__ + (j - 1) * p - icount + p + 1] * work[j];
		}
		x[i__] = ae + parm[i__ + 1];
	}
	return CDFLIB_OK;
}

