/*
 * Copyright (C) 2012 - Michael Baudin
 * Copyright (C) Bruno Pincon
 *
 * This file must be used under the terms of the CeCILL.
 * This source file is licensed as described in the file COPYING, which
 * you should have received as part of this distribution.  The terms
 * are also available at
 * http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
 *
 */

#include <math.h>
#include "cdflib.h"
#include "cdflib_private.h"

int cdflib_geornd(double pr, double *x)
{
	static double p_save = 1.0, ln_1_m_p = 0.0;
	double u;
	int status;
	double logu;

	status=cdflib_geoCheckParams("cdflib_geornd",pr);
	if (status!=CDFLIB_OK)
	{
		return status;
	}
	if ( pr == 1 ) 
	{
		*x=1.0;
		return CDFLIB_OK;
	}
	else if ( pr != p_save )   
	{
		/* => recompute log(1-pr) */
		p_save = pr; 
		ln_1_m_p = cdflib_log1p(-pr);
	}

	u = -cdflib_randgenerate();
	logu = cdflib_log1p(u);
	*x = floor( logu/ln_1_m_p);
	return CDFLIB_OK;
}      

