/*
 * Copyright (C) 2012 - Michael Baudin
 *
 * This file must be used under the terms of the CeCILL.
 * This source file is licensed as described in the file COPYING, which
 * you should have received as part of this distribution.  The terms
 * are also available at
 * http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
 *
 */

#include "cdflib.h"
#include "cdflib_private.h"

int cdflib_setgmn(double *meanv, double *covm, int *ldcovm, int *p, double *parm, int *ierr)
{
	int covm_dim1, covm_offset, i__1, i__2;

	static int i__, j;
	static int info;
	static int icount;
	
	// TODO : check the arguments

	/*     JJV changed this routine to take leading dimension of COVM */
	/*     JJV argument and pass it to cdflib_spofa, making it easier to use */
	/*     JJV if the COVM which is used is contained in a larger matrix */
	/*     JJV and to make the routine more consistent with LINPACK. */
	/*     JJV Changes are in comments, declarations, and the call to cdflib_spofa. */

	/*     .. Array Arguments .. */
	/*      DOUBLE PRECISION covm(p,p),meanv(p),parm(p* (p+3)/2+1) */

	/*     TEST THE INPUT */

	/* Parameter adjustments */
	--parm;
	covm_dim1 = *ldcovm;
	covm_offset = 1 + covm_dim1;
	covm -= covm_offset;
	--meanv;

	/* Function Body */
	parm[1] = (double) (*p);

	/*     PUT P AND MEANV INTO PARM */

	i__1 = *p + 1;
	for (i__ = 2; i__ <= i__1; ++i__) {
		parm[i__] = meanv[i__ - 1];
	}

	/*      Cholesky decomposition to find A s.t. trans(A)*(A) = COVM */

	/*      CALL cdflib_spofa(covm,p,p,info) */
	cdflib_spofa(&covm[covm_offset], ldcovm, p, &info);
	*ierr = 0;
	if (! (info != 0)) {
		goto L30;
	}
	cdflib_messageprint("SET Generate Multivariate Normal Random Number: COV not positive definite");
	*ierr = 1;
	return CDFLIB_OK;
L30:
	icount = *p + 1;

	/*     PUT UPPER HALF OF A, WHICH IS NOW THE CHOLESKY FACTOR, INTO PARM */
	/*          COVM(1,1) = PARM(P+2) */
	/*          COVM(1,2) = PARM(P+3) */
	/*                    : */
	/*          COVM(1,P) = PARM(2P+1) */
	/*          COVM(2,2) = PARM(2P+2)  ... */

	i__1 = *p;
	for (i__ = 1; i__ <= i__1; ++i__) {
		i__2 = *p;
		for (j = i__; j <= i__2; ++j) {
			++icount;
			parm[icount] = covm[i__ + j * covm_dim1];
		}
	}
	return CDFLIB_OK;
}

